# Generated by Django 5.2b1 on 2025-03-13 04:44

import django.contrib.auth.models
import django.contrib.auth.validators
import django.db.models.deletion
import django.utils.timezone
import uuid6
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0012_alter_user_first_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='TimeZone',
            fields=[
                ('uuid_pk_v7', models.UUIDField(default=uuid6.uuid7, editable=False, primary_key=True, serialize=False)),
                ('identifier', models.CharField(help_text='America/New_York', max_length=20, unique=True)),
                ('tz_type', models.PositiveSmallIntegerField(choices=[(1, 'canonical'), (2, 'link (alias)'), (3, 'link, previously canonical')], default=1)),
                ('src_file', models.PositiveSmallIntegerField(choices=[(1, 'factory'), (2, 'etcetera'), (3, 'backward'), (4, 'europe'), (5, 'asia'), (6, 'australasia'), (7, 'northamerica'), (8, 'southamerica'), (9, 'africa'), (10, 'antarctica')], default=7)),
                ('aliases', models.CharField(blank=True, help_text='US/Eastern, EST5EDT', max_length=30, null=True, unique=True)),
            ],
            options={
                'verbose_name': 'IANA time zone',
                'verbose_name_plural': 'IANA time zones',
                'ordering': ['identifier'],
            },
        ),
        migrations.CreateModel(
            name='TZAbbreviation',
            fields=[
                ('uuid_pk_v7', models.UUIDField(default=uuid6.uuid7, editable=False, primary_key=True, serialize=False)),
                ('identifier', models.CharField(help_text='Central European Summer Time', max_length=30, unique=True)),
                ('offset', models.CharField(help_text='+02:00', max_length=6)),
                ('abbr', models.CharField(help_text='CEST', max_length=4, unique=True)),
            ],
            options={
                'verbose_name': 'Time zone abbreviation',
                'verbose_name_plural': 'Tme zone abbreviations',
                'ordering': ['offset'],
            },
        ),
        migrations.CreateModel(
            name='User',
            fields=[
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('uuid_pk_v7', models.UUIDField(default=uuid6.uuid7, editable=False, primary_key=True, serialize=False)),
                ('surname', models.CharField(blank=True, max_length=150)),
                ('given_names', models.CharField(blank=True, max_length=150)),
                ('surname_first', models.BooleanField(default=False)),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='Staff')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='Super')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.permission', verbose_name='user permissions')),
                ('timezone', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='vto_core.timezone')),
            ],
            options={
                'ordering': ['-is_superuser', 'username'],
            },
            managers=[
                ('objects', django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.AddField(
            model_name='timezone',
            name='dst',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='rn_tz_abbr_dst', to='vto_core.tzabbreviation'),
        ),
        migrations.AddField(
            model_name='timezone',
            name='std',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='rn_tz_abbr_std', to='vto_core.tzabbreviation'),
        ),
    ]



from django.conf import settings
from django.contrib import admin, messages
from django.contrib.admin.options import IS_POPUP_VAR
from django.contrib.admin.utils import unquote
from django.contrib.auth import update_session_auth_hash
from django.contrib.auth.forms import (
    AdminPasswordChangeForm,
    AdminUserCreationForm,
    UserChangeForm,
)
# from django.contrib.auth.models import User  # Group
from django.core.exceptions import PermissionDenied
from django.db import router, transaction
from django.http import Http404, HttpResponseRedirect
from django.template.response import TemplateResponse
from django.urls import path, reverse
from django.utils.decorators import method_decorator
from django.utils.html import escape
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.debug import sensitive_post_parameters

from .models import TZAbbreviation, TimeZone, User

csrf_protect_m = method_decorator(csrf_protect)
sensitive_post_parameters_m = method_decorator(sensitive_post_parameters())


@admin.register(TZAbbreviation)
class TZAbbrAdmin(admin.ModelAdmin):
    list_display = [
        'offset',
        'abbr',
        'identifier',
    ]
    list_display_links = ['identifier']
    search_fields = [
        'identifier',
        'abbr',
    ]

@admin.register(TimeZone)
class TZAdmin(admin.ModelAdmin):
    autocomplete_fields = [
        'std',
        'dst',
    ]
    list_display = [
        'identifier',
        'tz_type',
        'src_file',
        'std',
        'dst',
        'aliases',
    ]
    search_fields = [
        'identifier',
        'std__identifier',
        'dst__identifier',
    ]

'''
@admin.register(Group)
class GroupAdmin(admin.ModelAdmin):
    search_fields = ("name",)
    ordering = ("name",)
    filter_horizontal = ("permissions",)

    def formfield_for_manytomany(self, db_field, request=None, **kwargs):
        if db_field.name == "permissions":
            qs = kwargs.get("queryset", db_field.remote_field.model.objects)
            # Avoid a major performance hit resolving permission names which
            # triggers a content_type load:
            kwargs["queryset"] = qs.select_related("content_type")
        return super().formfield_for_manytomany(db_field, request=request, **kwargs)
'''

@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    add_form_template = "admin/auth/user/add_form.html"
    autocomplete_fields = ['timezone']
    change_user_password_template = None
    fieldsets = (
        (None, {"fields": ("username", "password")}),
        (
            _("Personal info"),
                {"fields": (
                    "timezone",
                    "surname",
                    "given_names",
                    "surname_first",
                    "email",
                )
            }
        ),
        (
            _("Permissions"),
                {"fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (_("Important dates"), {"fields": ("last_login", "date_joined")}),
    )
    add_fieldsets = (
        (
            None,
            {
                "classes": ("wide",),
                "fields": ("username", "usable_password", "password1", "password2"),
            },
        ),
    )
    form = UserChangeForm
    add_form = AdminUserCreationForm
    change_password_form = AdminPasswordChangeForm

    # CUSTOM ORDER
    list_display = (
        "timezone",          # added
        "is_staff",
        "is_superuser",      # added
        "username",
        "email",
        "surname",           # added
        "given_names",       # added
        "get_full_name"      # added

    )
    list_display_links = ['username']  # added

    list_filter = ("is_staff", "is_superuser", "is_active", "groups")
    search_fields = ("username", "surname", "given_names", "email")
    
    # ordering = ("-is_superuser", "username",)  # disabled in favor of model
    
    filter_horizontal = (
        "groups",
        "user_permissions",
    )

    # --------------------------------------------------------------------------
    # CUSTOM CHANGE LIST DECORATORS
    # these retain sortability, but they do not retain default widget
    # --------------------------------------------------------------------------

    '''
    @admin.display(description='Super', ordering='is_superuser')
    def rename_superuser(self, obj):
        return obj.is_superuser
    '''

    # --------------------------------------------------------------------------

    def get_fieldsets(self, request, obj=None):
        if not obj:
            return self.add_fieldsets
        return super().get_fieldsets(request, obj)

    def get_form(self, request, obj=None, **kwargs):
        """
        Use special form during user creation
        """
        defaults = {}
        if obj is None:
            defaults["form"] = self.add_form
        defaults.update(kwargs)
        return super().get_form(request, obj, **defaults)

    def get_urls(self):
        return [
            path(
                "<id>/password/",
                self.admin_site.admin_view(self.user_change_password),
                name="auth_user_password_change",
            ),
        ] + super().get_urls()

    def lookup_allowed(self, lookup, value, request):
        # Don't allow lookups involving passwords.
        return not lookup.startswith("password") and super().lookup_allowed(
            lookup, value, request
        )

    @sensitive_post_parameters_m
    @csrf_protect_m
    def add_view(self, request, form_url="", extra_context=None):
        if request.method in ("GET", "HEAD", "OPTIONS", "TRACE"):
            return self._add_view(request, form_url, extra_context)

        with transaction.atomic(using=router.db_for_write(self.model)):
            return self._add_view(request, form_url, extra_context)

    def _add_view(self, request, form_url="", extra_context=None):
        # It's an error for a user to have add permission but NOT change
        # permission for users. If we allowed such users to add users, they
        # could create superusers, which would mean they would essentially have
        # the permission to change users. To avoid the problem entirely, we
        # disallow users from adding users if they don't have change
        # permission.
        if not self.has_change_permission(request):
            if self.has_add_permission(request) and settings.DEBUG:
                # Raise Http404 in debug mode so that the user gets a helpful
                # error message.
                raise Http404(
                    'Your user does not have the "Change user" permission. In '
                    "order to add users, Django requires that your user "
                    'account have both the "Add user" and "Change user" '
                    "permissions set."
                )
            raise PermissionDenied
        if extra_context is None:
            extra_context = {}
        username_field = self.opts.get_field(self.model.USERNAME_FIELD)
        defaults = {
            "auto_populated_fields": (),
            "username_help_text": username_field.help_text,
        }
        extra_context.update(defaults)
        return super().add_view(request, form_url, extra_context)

    @sensitive_post_parameters_m
    def user_change_password(self, request, id, form_url=""):
        user = self.get_object(request, unquote(id))
        if not self.has_change_permission(request, user):
            raise PermissionDenied
        if user is None:
            raise Http404(
                _("%(name)s object with primary key %(key)r does not exist.")
                % {
                    "name": self.opts.verbose_name,
                    "key": escape(id),
                }
            )
        if request.method == "POST":
            form = self.change_password_form(user, request.POST)
            if form.is_valid():
                # If disabling password-based authentication was requested
                # (via the form field `usable_password`), the submit action
                # must be "unset-password". This check is most relevant when
                # the admin user has two submit buttons available (for example
                # when Javascript is disabled).
                valid_submission = (
                    form.cleaned_data["set_usable_password"]
                    or "unset-password" in request.POST
                )
                if not valid_submission:
                    msg = gettext("Conflicting form data submitted. Please try again.")
                    messages.error(request, msg)
                    return HttpResponseRedirect(request.get_full_path())

                user = form.save()
                change_message = self.construct_change_message(request, form, None)
                self.log_change(request, user, change_message)
                if user.has_usable_password():
                    msg = gettext("Password changed successfully.")
                else:
                    msg = gettext("Password-based authentication was disabled.")
                messages.success(request, msg)
                update_session_auth_hash(request, form.user)
                return HttpResponseRedirect(
                    reverse(
                        "%s:%s_%s_change"
                        % (
                            self.admin_site.name,
                            user._meta.app_label,
                            user._meta.model_name,
                        ),
                        args=(user.pk,),
                    )
                )
        else:
            form = self.change_password_form(user)

        fieldsets = [(None, {"fields": list(form.base_fields)})]
        admin_form = admin.helpers.AdminForm(form, fieldsets, {})

        if user.has_usable_password():
            title = _("Change password: %s")
        else:
            title = _("Set password: %s")
        context = {
            "title": title % escape(user.get_username()),
            "adminForm": admin_form,
            "form_url": form_url,
            "form": form,
            "is_popup": (IS_POPUP_VAR in request.POST or IS_POPUP_VAR in request.GET),
            "is_popup_var": IS_POPUP_VAR,
            "add": True,
            "change": False,
            "has_delete_permission": False,
            "has_change_permission": True,
            "has_absolute_url": False,
            "opts": self.opts,
            "original": user,
            "save_as": False,
            "show_save": True,
            **self.admin_site.each_context(request),
        }

        request.current_app = self.admin_site.name

        return TemplateResponse(
            request,
            self.change_user_password_template
            or "admin/auth/user/change_password.html",
            context,
        )

    def response_add(self, request, obj, post_url_continue=None):
        """
        Determine the HttpResponse for the add_view stage. It mostly defers to
        its superclass implementation but is customized because the User model
        has a slightly different workflow.
        """
        # We should allow further modification of the user just added i.e. the
        # 'Save' button should behave like the 'Save and continue editing'
        # button except in two scenarios:
        # * The user has pressed the 'Save and add another' button
        # * We are adding a user in a popup
        if "_addanother" not in request.POST and IS_POPUP_VAR not in request.POST:
            request.POST = request.POST.copy()
            request.POST["_continue"] = 1
        return super().response_add(request, obj, post_url_continue)



'''
REFERENCES:

https://github.com/django/django/blob/5a1cae3a5675c5733daf5949759476d65aa0e636/django/contrib/auth/models.py#L471C1-L475C6
'''

# import zoneinfo
from django.contrib.auth.models import AbstractUser
from django.core.validators import MaxLengthValidator
from django.db import models
from django.db.models import PositiveSmallIntegerField as PSInt
from djinntoux.abstract.ab_mod import UUIDpk7
# from timezone_field import TimeZoneField

'''
    more info:
https://en.wikipedia.org/wiki/List_of_tz_database_time_zones
'''


class TZAbbreviation(UUIDpk7):

    identifier = models.CharField(max_length=30, unique=True,
        help_text='Central European Summer Time')

    offset = models.CharField(max_length=6, help_text='+02:00')

    abbr = models.CharField(max_length=4, unique=True, help_text='CEST')

    class Meta:
        ordering = ['offset']
        verbose_name = 'Time zone abbreviation'
        verbose_name_plural = 'Tme zone abbreviations'

    def __str__(self):
        return self.abbr


class TimeZone(UUIDpk7):

    identifier = models.CharField(max_length=20, unique=True,
        help_text='America/New_York')

    class TZType(models.IntegerChoices):
        CAN = 1, 'canonical'
        LNK = 2, 'link (alias)'
        LKP = 3, 'link, previously canonical'

    tz_type = PSInt(choices=TZType.choices, default=TZType.CAN)

    class SrcFile(models.IntegerChoices):
        FAC = 1, 'factory'
        ETC = 2, 'etcetera'
        BAC = 3, 'backward'
        EUR = 4, 'europe'
        ASI = 5, 'asia'
        AUS = 6, 'australasia'
        NAM = 7, 'northamerica'
        SAM = 8, 'southamerica'
        AFR = 9, 'africa'
        ANT = 10, 'antarctica'

    src_file = PSInt(choices=SrcFile.choices, default=SrcFile.NAM)

    aliases = models.CharField(max_length=30, unique=True,
        help_text='US/Eastern, EST5EDT', null=True, blank=True)

    std = models.ForeignKey('TZAbbreviation', on_delete=models.PROTECT,
        related_name='rn_tz_abbr_std')

    # nullable, as some zones may not observe DST
    dst = models.ForeignKey('TZAbbreviation', on_delete=models.PROTECT,
        related_name='rn_tz_abbr_dst', null=True, blank=True)

    class Meta:
        ordering = ['identifier']
        verbose_name = 'IANA time zone'
        verbose_name_plural = 'IANA time zones'

    def __str__(self):
        return self.identifier


class User(UUIDpk7, AbstractUser):
    first_name = None  # type: ignore[assignment]
    last_name = None  # type: ignore[assignment]

    surname = models.CharField(blank=True, max_length=150)
    given_names = models.CharField(blank=True, max_length=150)
    surname_first = models.BooleanField(default=False)

    # timezone = TimeZoneField(default='America/New_York')
    # timezone = models.CharField(max_length=40, default='America/New_York')
    timezone = models.ForeignKey('TimeZone', on_delete=models.PROTECT,
        null=True, blank=True)

    # override verbose name in admin change list
    # do I need to make a new migration?
    is_staff = models.BooleanField(verbose_name='Staff', default=False,
        help_text='Designates whether the user can log into this admin site.')

    # override verbose name in admin change list
    # do I need to make a new migration?
    is_superuser = models.BooleanField(verbose_name='Super', default=False,
        help_text='Designates that this user has all permissions without explicitly assigning them.')

    class Meta:
        ordering = ['-is_superuser', 'username']  # added

    def get_short_name(self):
        """Return the short name for the user."""
        return self.given_names

    def get_full_name(self):
        """
        Return the first_name plus the last_name, with a space in between.
        """
        if self.surname_first:
            full_name = "%s %s" % (self.surname, self.given_names)
        else:
            full_name = "%s %s" % (self.given_names, self.surname)
        return full_name.strip()